package cxa.components.source;

import org.apache.avalon.framework.component.ComponentManager;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.ResourceNotFoundException;
import org.apache.cocoon.components.parser.Parser;
import org.apache.cocoon.environment.ModifiableSource;
import org.apache.cocoon.environment.Source;
import org.apache.cocoon.xml.XMLConsumer;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.*;

public class ZipSource implements Source {

    private String systemId;
    private Source archive;
    private ComponentManager manager;
    private String documentName;

    public ZipSource(Source archive,
                     String documentName,
                     ComponentManager manager)
    throws IOException {
        this.manager  = manager;
        this.systemId = archive.getSystemId() + '/' + documentName;
        this.archive  = archive;
        this.documentName = documentName;
    }

    protected void update() {
        if (this.archive instanceof ModifiableSource) {
            ((ModifiableSource)this.archive).refresh();
        }
    }

    public long getLastModified() {
        this.update();
        return this.archive.getLastModified();
    }

    public long getContentLength() {
        return -1;
    }

    public InputStream getInputStream()
    throws IOException, ProcessingException {
        this.update();

        ZipInputStream zipStream = new ZipInputStream( this.archive.getInputStream() );
        ZipEntry document = null;
        boolean found = false;
        do {
            document = zipStream.getNextEntry();
            if (document != null) {
                if ( document.getName().equals(this.documentName) ) {
                    found = true;
                } else {
                    // go to next entry
                    zipStream.closeEntry();
                }
            }
        } while (document != null && found == false);

        if (document == null)
            throw new ResourceNotFoundException("The document " + documentName + " is not in the archive " + this.archive.getSystemId());

        // now we will extract the document and write it into a byte array
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        byte[] buffer = new byte[8192];
        int length = -1;
        while ( zipStream.available() > 0 ) {
            length = zipStream.read(buffer, 0, 8192);
            if ( length > 0 ) {
                baos.write(buffer, 0, length);
            }
        }
        zipStream.close();
        baos.flush();

        // return an input stream
        return new ByteArrayInputStream( baos.toByteArray() );
    }

    public String getSystemId() {
        return this.systemId;
    }

    public InputSource getInputSource()
    throws IOException, ProcessingException {
        InputSource newObject = new InputSource(this.getInputStream());
        newObject.setSystemId(this.systemId);
        return newObject;
    }

    public void toSAX(ContentHandler handler)
    throws SAXException, ProcessingException
    {
        Parser parser = null;
        try {
            parser = (Parser)this.manager.lookup(Parser.ROLE);

            if (handler instanceof XMLConsumer) {
                parser.setConsumer((XMLConsumer)handler);
            } else {
                parser.setContentHandler(handler);
                if (handler instanceof LexicalHandler) {
                    parser.setLexicalHandler((LexicalHandler)handler);
                }
            }
            parser.parse(this.getInputSource());
        } catch (ProcessingException e){
            // Preserve original exception
            throw e;
        } catch (SAXException e) {
            // Preserve original exception
            throw e;
        } catch (Exception e){
            throw new ProcessingException("Exception during processing of "
                                          + this.systemId, e);
        } finally {
            if (parser != null) this.manager.release(parser);
        }
    }

    public void recycle()
    {
    }
}
