package cxa.generation;

import org.apache.avalon.excalibur.pool.Recyclable;
import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.ResourceNotFoundException;
import org.apache.cocoon.caching.Cacheable;
import org.apache.cocoon.caching.CacheValidity;
import org.apache.cocoon.caching.TimeStampCacheValidity;
import org.apache.cocoon.components.parser.Parser;
import org.apache.cocoon.environment.Source;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.generation.ComposerGenerator;
import org.apache.cocoon.util.HashUtil;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

public class CacheableZipGenerator
  extends ComposerGenerator
  implements Cacheable, Recyclable {

    /** The source for the archive */
    protected Source archive;

    /** The document name */
    protected String documentName;

    /**
     * Setup
     */
    public void setup(SourceResolver resolver,
                      Map            objectModel,
                      String         src,
                      Parameters     par)
    throws ProcessingException, SAXException, IOException {
        super.setup(resolver, objectModel, src, par);

        int position = this.source.indexOf(".zip");
        if (position == -1) {
            throw new ProcessingException("The URI does not point to a ZIP archive.");
        }
        String zipURI = this.source.substring(0, position + 4);
        this.documentName = this.source.substring(position + 5);

        if ( this.getLogger().isDebugEnabled() ) {
            this.getLogger().debug("Reading " + documentName + " from zip archive " + zipURI);
        }
        this.archive = resolver.resolve(zipURI );
    }

    /**
     * Recycle
     */
    public void recycle() {
        if (this.archive != null) {
            this.archive.recycle();
            this.archive = null;
       }
    }

    /**
     * Generate the unique key.
     * This key must be unique inside the space of this component.
     * This method must be invoked before the generateValidity() method.
     *
     * @return The generated key or <code>0</code> if the component
     *              is currently not cacheable.
     */
    public long generateKey() {
        if (this.archive.getLastModified() != 0) {
            return HashUtil.hash(this.archive.getSystemId() + '/' + this.documentName);
        }
        return 0;
    }

    /**
     * Generate the validity object.
     * Before this method can be invoked the generateKey() method
     * must be invoked.
     *
     * @return The generated validity object or <code>null</code> if the
     *         component is currently not cacheable.
     */
    public CacheValidity generateValidity() {
        if (this.archive.getLastModified() != 0) {
            return new TimeStampCacheValidity(this.archive.getLastModified());
        }
        return null;
   }

    /**
     * Generates the requested resource.
     */
    public void generate()
    throws SAXException, IOException, ProcessingException {
        ZipInputStream zipStream = new ZipInputStream( this.archive.getInputStream() );
        ZipEntry document = null;
        boolean found = false;
        do {
            document = zipStream.getNextEntry();
            if (document != null) {
                if ( document.getName().equals(this.documentName) ) {
                    found = true;
                } else {
                    // go to next entry
                    zipStream.closeEntry();
                }
            }
        } while (document != null && found == false);

        if (document == null)
            throw new ResourceNotFoundException("The document " + this.documentName + " is not in the archive " + this.archive.getSystemId());

        // now we will extract the document and write it into a byte array
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        byte[] buffer = new byte[8192];
        int length = -1;

        while ( zipStream.available() > 0 ) {
            length = zipStream.read(buffer, 0, 8192);
            if ( length > 0 ) {
                baos.write(buffer, 0, length);
            }
        }
        zipStream.close();
        baos.flush();

        // now we create the InputSource object for the parser
        ByteArrayInputStream is = new ByteArrayInputStream( baos.toByteArray() );
        Parser parser = null;
        try {
            InputSource source = new InputSource(is);

            parser = (Parser) this.manager.lookup (Parser.ROLE);
            parser.setConsumer( this.xmlConsumer );
            parser.parse( source );
        } catch (ComponentException ce) {
            throw new ProcessingException("Component Parser not found.", ce);
        } finally {
            this.manager.release( parser );
            is.close();
        }
    }

}
